SystemOrganization addCategory: #'System-PragmaCollector'!

Announcement subclass: #PragmaCollectorReset
	instanceVariableNames: 'collector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-PragmaCollector'!

!PragmaCollectorReset commentStamp: 'alain.plantec 10/20/2009 11:14' prior: 0!
A PragmaCollectorReset is an announce which is announced by a PragamColllector when it is reset.
see PragmaCollector>>reset.

Instance Variables
	collector:		<PragmaCollector>

collector
	- the collector which is reset
!

!PragmaCollectorReset class methodsFor: 'as yet unclassified' stamp: 'alain.plantec 10/19/2009 11:45'!
collector: aCollector
	^ self new collector: aCollector! !

!PragmaCollectorReset methodsFor: 'as yet unclassified' stamp: 'alain.plantec 10/19/2009 11:45'!
collector: aCollector
	collector := aCollector! !

Object subclass: #PragmaCollector
	instanceVariableNames: 'announcer collected filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-PragmaCollector'!

!PragmaCollector commentStamp: 'AlainPlantec 11/28/2009 01:09' prior: 0!
A PragmaCollector is used in order to collect some Pragma instances. A PragmaCollector makes use of SystemChangeNotifier event notifications in order to maintain its contents up-to-date according to its filter: when a method is added, removed or updated, if the method is defined with a pragma which is acceptable according to its filter, then the collector contents is updated. A PragmaCollector makes use of an announcer in order to notify all registered listeners when a pragma is added, removed or updated. A PragmaAnnouncement is announced when a Pragma is added, removed or updated. Corresponding announcement classes are, respectiveley, PragmaAdded, PragmaRemoved and PragmaUpdated. 

Explore the result of the expression below. In the collected instance variable should be stored all pragmas of the system:
---------------------------
(PragmaCollector filter: [:pragma | true]) reset
---------------------------

In the following example, collected pragma are thoses with the 'primitive:' keyword (<primitive:>)
---------------------------
(PragmaCollector filter: [:prg | prg keyword = 'primitive:']) reset
---------------------------

Instance Variables	
	announcer:		<Announcer>	
	collected:		<Collection>
	filter:			<Block or MessageSend>
				
announcer
	the announcer which is used to announce the adding, the removing or the updating of a method with an acceptable pragma declaration

collected		
	the current collection of Pragma
	
filter
	a block or a message send which is used in order to filter the pragma. This is a one argument valuable. When evaluated, the candidate pragam is passed as argument and the result must be a boolean. 
	!

!PragmaCollector class methodsFor: 'utilities' stamp: 'MarcusDenker 7/12/2012 17:59'!
allSystemPragmas
	^ (Array
		streamContents: [:stream | SystemNavigation new
				allBehaviorsDo: [:behavior | Pragma
						withPragmasIn: behavior
						do: [:pragma | stream nextPut: pragma]]]) ! !

!PragmaCollector class methodsFor: 'instance creation' stamp: 'alain.plantec 10/19/2009 16:16'!
filter: aOneArgValuable
	^ self new filter: aOneArgValuable! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:54'!
addPragma: aPragma
	"if aPragma is to be kept, then add it and announce its adding"
	(self keepPragma: aPragma)
		ifTrue: [self collected add: aPragma.
			self announce: (PragmaAdded pragma: aPragma)]! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 10/19/2009 10:55'!
addedEventOccurs: anEvent
	"method adding event occured: if the 
	concerned method contains a pragma then 
	try to update myself with it"
	Pragma
		withPragmasIn: anEvent itemClass
		do: [:pragma | pragma selector = anEvent itemSelector
				ifTrue: [self addPragma: pragma]] ! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
announce: anAnnouncement
	"see Announcements packages"
	self announcer ifNotNil: [announcer announce: anAnnouncement]! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 15:26'!
announcer
	^ announcer! !

!PragmaCollector methodsFor: 'system changes' stamp: 'StephaneDucasse 6/17/2012 17:40'!
classRemovedEventOccurs: anEvent
	"a class has been removed: first see if the class is not my class
	because then I must be unplugged from system event notifications"
	anEvent removedClass = self class
		ifTrue: [^ self noMoreNotifications].
	"remove all handler which are from the removed class"
	(self pragmasOfClass: anEvent removedClass class)
		do: [:handler | self removePragma: handler] ! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
collect: aBlock
	^ self collected collect: aBlock
	! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 10/20/2009 13:15'!
collected
	^ collected ifNil: [collected := OrderedCollection new] ! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
detect: aBlock
	^ self collected detect: aBlock ifNone:[]
	! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
do: aBlock
	self collected do: aBlock
	! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 10/19/2009 10:59'!
filter
	^ filter ifNil: [filter := [:prg | true]]! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 10/19/2009 11:00'!
filter: aOneArgValuable
	filter := aOneArgValuable! !

!PragmaCollector methodsFor: 'testing' stamp: 'alain.plantec 9/18/2009 15:26'!
ifNotEmpty: aBlock
	self collected ifNotEmpty: aBlock! !

!PragmaCollector methodsFor: 'initializing' stamp: 'alain.plantec 9/18/2009 15:26'!
initialize
	super initialize.
	announcer := Announcer new.
	self installSystemNotifications! !

!PragmaCollector methodsFor: 'system changes' stamp: 'EstebanLorenzano 8/3/2012 14:29'!
installSystemNotifications
	"Allows myself to be kept up-to-date regarding system changes"
	SystemAnnouncer uniqueInstance weak
			on: ClassRemoved send: #classRemovedEventOccurs: to: self;
			on: MethodRemoved send: #removedEventOccurs: to: self;
			on: MethodAdded send: #addedEventOccurs: to: self;
			on: MethodModified send: #modifiedEventOccurs: to: self! !

!PragmaCollector methodsFor: 'testing' stamp: 'alain.plantec 9/18/2009 15:26'!
isEmpty
	^ self collected isEmpty! !

!PragmaCollector methodsFor: 'testing' stamp: 'alain.plantec 9/18/2009 15:26'!
isNotEmpty
	^ self collected isNotEmpty! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:58'!
keepPragma: aPragma
	^ self filter value: aPragma
! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 10/19/2009 10:55'!
modifiedEventOccurs: anEvent 
	"
	a method has been updated: try to update an handler:
	1 - I already have one for the method, then
	do nothing, 
	2 - I do not have one but method has an acceptable
	pragma in it, then I try to add a new handler
	3 - I have one but changed method has no more
	acceptable pragma in it, then the handler is removed.
	"
	Pragma
		withPragmasIn: anEvent itemClass
		do: [:pragma | pragma selector = anEvent itemSelector
				ifTrue: [(self 
								detect: [:oldprag | oldprag selector = pragma selector
										and: [oldprag methodClass = anEvent itemClass]])
						ifNotNil: [:oldprag | ^ self updatePragma: oldprag].
					^ self addPragma: pragma]].
	"No pragma but an handler for the method"
	(self pragmaWithSelector: anEvent itemSelector inClass: anEvent itemClass)
		ifNotNil: [:found | self removePragma: found]! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 9/18/2009 15:26'!
noMoreAnnounceWhile: aBlock
	"unplug the announcer during aBlock"
	| oldAnnouncer |
	[oldAnnouncer := announcer.
	announcer := nil.
	aBlock value]
		ensure: [announcer := oldAnnouncer]! !

!PragmaCollector methodsFor: 'system changes' stamp: 'EstebanLorenzano 8/3/2012 14:08'!
noMoreNotifications
	"Do not receiver any system change notification anymore"
	SystemAnnouncer uniqueInstance unsubscribe: self.! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 9/18/2009 15:26'!
noMoreNotificationsWhile: aBlock
	"don not receive any system change notification during aBloc"
	self noMoreNotifications.
	[ aBlock value ] ensure: [ self installSystemNotifications ]! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
noneSatisfy: aBlock
	^ self collected noneSatisfy: aBlock! !

!PragmaCollector methodsFor: 'querying' stamp: 'alain.plantec 9/18/2009 15:44'!
pragmaWithSelector: aSelector inClass: aClass 
	"return the handler corresponding to a pragma method which selector is aSelector in class aClass"
	^ (self detect: [:prag | prag methodClass = aClass and: [prag  selector = aSelector]])
	! !

!PragmaCollector methodsFor: 'querying' stamp: 'alain.plantec 9/18/2009 15:30'!
pragmasOfClass: aClass 
	"return all handlers of class aClass"
	^ self	select: [:prag | prag methodClass = aClass ]! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
reject: aBlock
	^ self collected reject: aBlock ! !

!PragmaCollector methodsFor: 'dependents access' stamp: 'alain.plantec 9/18/2009 22:09'!
release
	self noMoreNotifications.
	announcer := nil.
	collected := nil.
	super release! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:48'!
removePragma: aPragma 
	"remove an handler an announce it"
	self collected remove: aPragma ifAbsent: [].
	self announce: (PragmaRemoved pragma: aPragma)! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 9/18/2009 15:40'!
removedEventOccurs: anEvent
	"a method has been removed, remove any corresponding handler if found"
	(self
				detect: [:prag | prag selector = anEvent itemSelector
						and: [prag methodClass = anEvent itemClass ]])
		ifNotNil: [:found | self removePragma: found]! !

!PragmaCollector methodsFor: 'initializing' stamp: 'alain.plantec 10/19/2009 11:46'!
reset
	"reinitialize current system settings"
	self
		noMoreAnnounceWhile: [self collected copy
				do: [:pragma | self removePragma: pragma].
			self class allSystemPragmas
				do: [:pragma | self addPragma: pragma]].
	self announce: (PragmaCollectorReset collector: self)! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
select: aBlock
	^ self collected select: aBlock
	! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
subscribe: anAnnouncement send: aSelector to: anObject
	"see Announcements packages"
	self announcer subscribe: anAnnouncement send: aSelector to: anObject

! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
unsubscribe: anObject
	"see Announcements packages"
	self announcer unsubscribe: anObject

! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:49'!
updatePragma: aPragma
	"only announce that the pragma has been updated"
	self announcer announce: (PragmaUpdated pragma: aPragma)! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
whenChangedSend: aSelector to: anObject
	"record a change listener"
	self subscribe: PragmaAnnouncement send: aSelector to: anObject
! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 10/19/2009 10:42'!
whenResetSend: aSelector to: anObject
	"record a change listener"
	self subscribe: PragmaCollectorReset send: aSelector to: anObject
! !
